// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Projections/Beam.hh"

#define IS_BHADRON_PDGID(id) ( ((abs(id)/100)%10 == 5) || (abs(id) >= 5000 && abs(id) <= 5999) )
#define IS_PARTON_PDGID(id) ( abs(id) <= 100 && abs(id) != 22 && (abs(id) < 11 || abs(id) > 18) )

namespace Rivet {


  /// @brief DELPHI b-fragmentation measurement for weak B-hadron decays
  /// @author Simone Amoroso
  class DELPHI_2011_I890503 : public Analysis {
  public:

    /// Constructor
    DEFAULT_RIVET_ANALYSIS_CTOR(DELPHI_2011_I890503);


    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      // Initialise and register projections
      declare(Beam(), "Beams");
      declare(ChargedFinalState(), "FS");

      // Book histograms
      _histXbweak     = bookHisto1D(1, 1, 1);
      _histMeanXbweak = bookProfile1D(2, 1, 1);

    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {

      const FinalState& fs = apply<FinalState>(event, "FS");
      const size_t numParticles = fs.particles().size();

      // Even if we only generate hadronic events, we still need a cut on numCharged >= 2.
      if (numParticles < 2) {
	MSG_DEBUG("Failed ncharged cut");
	vetoEvent;
      }
      MSG_DEBUG("Passed ncharged cut");

      // Get event weight for histo filling
      const double weight = event.weight();

      // Get beams and average beam momentum
      const ParticlePair& beams = apply<Beam>(event, "Beams").beams();
      const double meanBeamMom = ( beams.first.p3().mod() +beams.second.p3().mod() ) / 2.0;
      MSG_DEBUG("Avg beam momentum = " << meanBeamMom);

      foreach (const GenParticle* p, particles(event.genEvent())) {
	const GenVertex* pv = p->production_vertex();
	const GenVertex* dv = p->end_vertex();
	if (IS_BHADRON_PDGID(p->pdg_id())) {
	  const double xp = p->momentum().e()/meanBeamMom;

	  // If the B-hadron has no B-hadron as a child, it decayed weakly:
	  if (dv) {
	    bool is_weak = true;
	    for (GenVertex::particles_out_const_iterator pp = dv->particles_out_const_begin() ;
		 pp != dv->particles_out_const_end() ; ++pp) {
	      if (IS_BHADRON_PDGID((*pp)->pdg_id())) is_weak = false;
	    }
	    if (is_weak) {
	      _histXbweak->fill(xp, weight);
	      _histMeanXbweak->fill(_histMeanXbweak->bin(0).xMid(), xp, weight);
	    }
	  }
	}
      }

    }


    /// Normalise histograms etc., after the run
    void finalize() {
      normalize(_histXbweak);
    }

    //@}


  private:

    Histo1DPtr _histXbweak;
    Profile1DPtr _histMeanXbweak;

  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(DELPHI_2011_I890503);


}
